@@ -1,8 +1,8 @@
 //#################################################################################################
 //# This file is part of RobustGaussianFittingLibrary, a free library WITHOUT ANY WARRANTY        # 
-//# Copyright: 2017-2020 LaTrobe University Melbourne, 2019-2020 Deutsches Elektronen-Synchrotron # 
+//# Authors: Alireza Sadri, DESY/CFEL,															  #
+//#          Marjan Hadian Jazi, LaTrobe University,                                              #
 //#################################################################################################
-// gcc -fPIC -shared -o RobustGausFitLib.c RobustGausFitLib.so
 
 #include "RGFLib.h"
 #include <stdio.h>
@@ -295,7 +295,7 @@ void RobustSingleGaussianVec(float* vec, float* modelParams,
     else if(N==2) {
         optIters = 0;
     }
-    if(N<12) {
+    if(N<5) {
         botkPerc = 0;
         MSSE_LAMBDA = 0;
     }
@@ -304,16 +304,13 @@ void RobustSingleGaussianVec(float* vec, float* modelParams,
     	if(theta == modelParams[0])
     		theta = NEGATIVE_MAX;
 
-        for(iter=0; iter<optIters-2; iter++) {
+        for(iter=0; iter<optIters; iter++) {
             for (i = 0; i < N; i++) {
                 errorVec[i].vecData  = fabs(vec[i] - theta);
                 errorVec[i].indxs = i;
             }
             quickSort(errorVec,0,N-1);
 
-            if(iter==optIters-1)
-            	botkPerc = 0;
-
             theta = 0;
             tmp = 0;
             for(i=(int)(N*botkPerc); i<(int)(N*topkPerc); i++) {
@@ -328,7 +325,6 @@ void RobustSingleGaussianVec(float* vec, float* modelParams,
 		weights = (float*) malloc(N * sizeof(float));
 		float* residual;
 		residual = (float*) malloc(N * sizeof(float));
-		//Final mean-shift touch (which is now outlier free), very expensive
 		estScale = 0;
 		for (i = 0; i < N; i++) {
 			residual[i]  = vec[i] - theta;
@@ -340,24 +336,40 @@ void RobustSingleGaussianVec(float* vec, float* modelParams,
 		estScale = sqrt(estScale/((int)(N*topkPerc)));
 
 		if(MSSE_LAMBDA>0) {
-			for (iter = 0; iter < 2; iter++) {
 				estScale = MSSEWeighted(residual, weights,
 						                N, MSSE_LAMBDA,
 										(int)(N*topkPerc),
 										minimumResidual);
-				theta = 0;
-				tmp = 0;
-				for(i=0; i<N; i++) {
-					if(fabs(residual[i])<MSSE_LAMBDA*estScale) {
-						theta += vec[i];
-						tmp++;
+				if(optIters>1) {
+					theta = 0;
+					tmp = 0;
+					for(i=0; i<N; i++) {
+						if(fabs(residual[i])<MSSE_LAMBDA*estScale) {
+							theta += vec[i];
+							tmp++;
+						}
+					}
+					theta = theta / tmp;
+					for (i = 0; i < N; i++) {
+						residual[i]  = vec[i] - theta;
+					}
+					estScale = MSSEWeighted(residual, weights,
+							                N, MSSE_LAMBDA,
+											(int)(N*topkPerc),
+											minimumResidual);
+					theta = 0;
+					tmp = 0;
+					for(i=0; i<N; i++) {
+						if(fabs(residual[i])<MSSE_LAMBDA*estScale) {
+							theta += vec[i];
+							tmp++;
+						}
+					}
+					theta = theta / tmp;
+					for (i = 0; i < N; i++) {
+						residual[i]  = vec[i] - theta;
 					}
 				}
-				theta = theta / tmp;
-				for (i = 0; i < N; i++) {
-					residual[i]  = vec[i] - theta;
-				}
-			}
         }
 		free(weights);
         free(residual);
@@ -506,31 +518,52 @@ void fitValue2Skewed(float* inVec, float* inWeights,
         theta = theta_new / tmp;
     }
     
-    //////////////// calculate the std by inliers  /////////////////
-    float *residuals;
-    residuals = (float*) malloc(N * sizeof(float));
+	float* residuals;
+	residuals = (float*) malloc(N * sizeof(float));
+	estScale = 0;
+	for (i = 0; i < N; i++) {
+		residuals[i]  = vec[i] - theta;
+		if(i<(int)(N*topkPerc)) {
+			estScale += (errorVec[i].vecData)*(errorVec[i].vecData);
+		}
+	}
+	estScale = sqrt(estScale/((int)(N*topkPerc)));
 
-    tmp = 0;
-    estScale = 0;
-    for (i = 0; i < N; i++) {
-        residuals[i]  = fabs(theta - vec[i]);
-        if(i<topk) {
-        	estScale += (errorVec[i].vecData)*(errorVec[i].vecData);
-        	tmp += weights[i];
-        }
-    }
-    estScale = sqrt(estScale/tmp);
-    if(MSSE_LAMBDA) {
-    	estScale = MSSEWeighted(residuals, weights, N, MSSE_LAMBDA, topk, minimumResidual);
-		theta = 0;
-		tmp = 0;
-		for(i=0; i<N; i++) {
-			if(fabs(weights[i]*residuals[i])<MSSE_LAMBDA*estScale) {
-				theta += weights[i]*vec[i];
-				tmp += weights[i];
+	if(MSSE_LAMBDA>0) {
+		estScale = MSSEWeighted(residuals, weights,
+								N, MSSE_LAMBDA,
+								(int)(N*topkPerc),
+								minimumResidual);
+		if(optIters>1) {
+			theta = 0;
+			tmp = 0;
+			for(i=0; i<N; i++) {
+				if(fabs(residuals[i])<MSSE_LAMBDA*estScale) {
+					theta += vec[i];
+					tmp++;
+				}
+			}
+			theta = theta / tmp;
+			for (i = 0; i < N; i++) {
+				residuals[i]  = vec[i] - theta;
+			}
+			estScale = MSSEWeighted(residuals, weights,
+									N, MSSE_LAMBDA,
+									(int)(N*topkPerc),
+									minimumResidual);
+			theta = 0;
+			tmp = 0;
+			for(i=0; i<N; i++) {
+				if(fabs(residuals[i])<MSSE_LAMBDA*estScale) {
+					theta += vec[i];
+					tmp++;
+				}
+			}
+			theta = theta / tmp;
+			for (i = 0; i < N; i++) {
+				residuals[i]  = vec[i] - theta;
 			}
 		}
-		theta = theta / tmp;
     }
 
 	modelParams[0] = theta;
@@ -873,7 +906,8 @@ void RSGImage(float* inImage, unsigned char* inMask, float* modelParamsMap,
                 unsigned int X, unsigned int Y, 
                 float topkPerc, float botkPerc,
                 float MSSE_LAMBDA, unsigned char stretch2CornersOpt,
-                unsigned char numModelParams, unsigned char optIters) {
+                unsigned char numModelParams, unsigned char optIters,
+				float minimumResidual) {
 
     float* x;
     x = (float*) malloc(winX*winY * sizeof(float));
@@ -894,23 +928,25 @@ void RSGImage(float* inImage, unsigned char* inMask, float* modelParamsMap,
 
             if(numModelParams==1) {
                 numElem = 0;
-                for(rCnt = pRowStart; rCnt < pRowEnd; rCnt++)
-                    for(cCnt = pClmStart; cCnt < pClmEnd; cCnt++)
+                for(rCnt = pRowStart; rCnt < pRowEnd; rCnt++) {
+                    for(cCnt = pClmStart; cCnt < pClmEnd; cCnt++) {
                         if(inMask[cCnt + rCnt*Y]) {
                             z[numElem] = inImage[cCnt + rCnt*Y];
                             numElem++;
                         }
-                if((int) (botkPerc*numElem)>12) {
-                    RobustSingleGaussianVec(z, mP_MOne, 0, numElem, 
-                                            topkPerc, botkPerc, 
-                                            MSSE_LAMBDA, optIters, 0);
-                    for(rCnt=pRowStart; rCnt<pRowEnd; rCnt++) {
-                        for(cCnt=pClmStart; cCnt<pClmEnd; cCnt++) {
-                            modelParamsMap[cCnt + rCnt*Y + 0*X*Y] = mP_MOne[0];
-                            modelParamsMap[cCnt + rCnt*Y + 1*X*Y] = mP_MOne[1];
-                        }
-                    }
-                }
+					}
+				}
+                mP_MOne[0] = 0;
+				mP_MOne[1] = 0;
+				RobustSingleGaussianVec(z, mP_MOne, 0, numElem, 
+										topkPerc, botkPerc, 
+										MSSE_LAMBDA, optIters, minimumResidual);
+				for(rCnt=pRowStart; rCnt<pRowEnd; rCnt++) {
+					for(cCnt=pClmStart; cCnt<pClmEnd; cCnt++) {
+						modelParamsMap[cCnt + rCnt*Y + 0*X*Y] = mP_MOne[0];
+						modelParamsMap[cCnt + rCnt*Y + 1*X*Y] = mP_MOne[1];
+					}
+				}
             }
         
             if(numModelParams==4) {
@@ -928,7 +964,8 @@ void RSGImage(float* inImage, unsigned char* inMask, float* modelParamsMap,
                     
                     RobustAlgebraicPlaneFitting(x, y, z, mP, mP,
                                                 numElem, topkPerc,
-                                                botkPerc, MSSE_LAMBDA, stretch2CornersOpt, 0, 12);
+                                                botkPerc, MSSE_LAMBDA, stretch2CornersOpt, 
+												minimumResidual, 12);
                     
                     for(rCnt=pRowStart; rCnt<pRowEnd; rCnt++) {
                         for(cCnt=pClmStart; cCnt<pClmEnd; cCnt++) {
@@ -964,7 +1001,8 @@ void RSGImage_by_Image_Tensor(float* inImage_Tensor, unsigned char* inMask_Tenso
                         unsigned int N, unsigned int X, unsigned int Y, 
                         float topkPerc, float botkPerc,
                         float MSSE_LAMBDA, unsigned char stretch2CornersOpt,
-                        unsigned char numModelParams, unsigned char optIters) {
+                        unsigned char numModelParams, unsigned char optIters,
+						float minimumResidual) {
 
     unsigned int frmCnt, rCnt, cCnt;
     float* modelParamsMap;
@@ -989,7 +1027,7 @@ void RSGImage_by_Image_Tensor(float* inImage_Tensor, unsigned char* inMask_Tenso
                  winX, winY, X, Y,
                  topkPerc, botkPerc,
                  MSSE_LAMBDA, stretch2CornersOpt,
-                 numModelParams, optIters);
+                 numModelParams, optIters, minimumResidual);
         
         for(rCnt=0; rCnt<X; rCnt++) {
             for(cCnt=0; cCnt<Y; cCnt++) {
@@ -1004,8 +1042,8 @@ void RSGImage_by_Image_Tensor(float* inImage_Tensor, unsigned char* inMask_Tenso
 }
 
 void fitBackgroundRadially(float* inImage, unsigned char* inMask, 
-                           float* modelParamsMap,
-                            unsigned int minRes, 
+                           float* modelParamsMap, float* vecMP,
+                           unsigned int minRes,
                            unsigned int maxRes, 
                            unsigned int shellWidth,
                            unsigned char includeCenter, 
@@ -1013,36 +1051,57 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
                            unsigned int X, unsigned int Y,
                            float topkPerc, float botkPerc, 
                            float MSSE_LAMBDA, 
-                           unsigned char optIters) {
+                           unsigned char optIters,
+						   float minimumResidual) {
 
     int r, maxR, shellCnt, pixX, pixY, i, j,pixInd, new_numElem;
     int shell_low, shell_high, numElem;
     float theta, sCnt, jump, X_Cent, Y_Cent;
     float mP[2];
     float* inVec;
+    float shellWidth_FSB, currentShell, area;
+    float squareSide;
     
-    if(minRes<1) minRes=1;
+    if(finiteSampleBias<1)		finiteSampleBias = 1;
+    if(minRes<finiteSampleBias)		minRes = (unsigned int) (finiteSampleBias/(2*M_PI));
 
     X_Cent = (int)(ceil(X/2));
     Y_Cent = (int)(ceil(Y/2));
 
+    squareSide = X_Cent;
+    if(squareSide > Y_Cent)		squareSide = Y_Cent;
+
     maxR = (int)(ceil(sqrt(X*X/4 + Y*Y/4)));
     float *resShells;
     resShells = (float*) malloc(maxR * sizeof(float));
-    for(i=0; i<maxR; i++)
-        resShells[i]=0;
+    for(i=0; i<maxR; i++)	resShells[i]=0;
 
     shellCnt = 0;
     if(includeCenter) {
-        resShells[shellCnt] = 1;
+        resShells[shellCnt] = 1;	//no radius 0
         shellCnt++;
     }
     resShells[shellCnt] = minRes;
 
-    while(resShells[shellCnt]<maxRes) {
-        shellCnt++;
-        resShells[shellCnt] = resShells[shellCnt-1] + shellWidth;
+    while(resShells[shellCnt] < maxRes) {
+    	shellCnt++;
+    	currentShell = resShells[shellCnt-1];
+        shellWidth_FSB = 0;
+        area = 0;
+        while( (area<finiteSampleBias) && ( currentShell+shellWidth_FSB < maxRes ) ) {
+			if(currentShell+shellWidth_FSB > squareSide) {
+				area += 4*(M_PI/2 - 2*(acos(squareSide/(currentShell+shellWidth_FSB))))*(currentShell+shellWidth_FSB);
+			}
+			else {
+				area += 2*M_PI*(currentShell + shellWidth_FSB);
+			}
+			shellWidth_FSB += 1;
+        }
+
+        if(shellWidth_FSB<shellWidth)	shellWidth_FSB = shellWidth;
+        resShells[shellCnt] = currentShell + shellWidth_FSB;
     }
+
     if(resShells[shellCnt]>maxRes) {
         resShells[shellCnt] = maxRes;
     }
@@ -1050,6 +1109,8 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
     for(i=0; i<shellCnt; i++) {
         shell_low = resShells[i];
         shell_high = resShells[i+1];
+
+        //Fill in the vector of resolution shell
         inVec = (float*) malloc((int)ceil( 2*M_PI*(shell_high-shell_low+1)*(shell_high+shell_low)/2 ) * sizeof(float));
         numElem = 0;
         for(r=shell_low; r<shell_high; r++) {
@@ -1063,7 +1124,7 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
                 }
             }
         }
-
+        //Downsample it if it is too big
         if(numElem > finiteSampleBias) {
             new_numElem = 0;
             jump = (float)numElem/((float)finiteSampleBias);
@@ -1071,10 +1132,10 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
                 inVec[new_numElem++] = inVec[(int)sCnt];
             numElem = new_numElem;
         }
-
+        //Get the model parameters
         if(optIters>0) {
             RobustSingleGaussianVec(inVec, mP, 0, numElem,
-                    topkPerc, botkPerc, MSSE_LAMBDA, optIters, 0);
+                    topkPerc, botkPerc, MSSE_LAMBDA, optIters, minimumResidual);
         }        
         else {
             mP[0] = 0;
@@ -1088,8 +1149,12 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
             mP[1] = sqrt(mP[1]);
         }
         free(inVec);
-        
+        //Fill the output with model parameters
+        //We will set four pixels around each radius as this shell
+        //This way no pixel will be forgotten
         for(r=shell_low; r<shell_high; r++) {
+			vecMP[r + 0*maxR] = mP[0];
+			vecMP[r + 1*maxR] = mP[1];
             for(theta = 0; theta < 2*M_PI; theta += 1.0/r) {
                 pixX = (int)((float)r*cos(theta) + X_Cent);
                 pixY = (int)((float)r*sin(theta) + Y_Cent);
