@@ -135,7 +135,7 @@ def fitValue(inVec,
                                                    MSSE_LAMBDA,
                                                    optIters,
                                                    minimumResidual)
-    return (modelParams[0], modelParams[1])
+    return modelParams
 
 def fitValue2Skewed(inVec, 
                     inWeights = None,
@@ -191,7 +191,71 @@ def fitValue2Skewed(inVec,
                                minimumResidual)
     return (modelParams[0], modelParams[1])
     
-    
+'''
+void medianOfFits(float *vec, float *weights, 
+                  float *modelParams, float theta, unsigned int N,
+                  float topkMin, float topkMax, unsigned int numSamples, float samplePerc,
+                  float MSSE_LAMBDA, unsigned char optIters, float minimumResidual) 
+'''
+def medianOfFits(inVec, 
+                 inWeights = None,
+                 topkMax = 0.7,
+                 topkMin = 0.3,
+                 numSamples = 50,
+                 samplePerc = 0.1,
+                 MSSE_LAMBDA = 3.0,
+                 modelValueInit = 0,
+                 optIters = 12,
+                 minimumResidual = 0):
+    """Fit a skewed bell shaped unimodal sharp density robustly:
+    The function works exactly the same as the fitValue, it fits a Gaussian to inVec robustly. Except that it accepts weights as well and returns the average and standard deviation of a skewed density, it reports the bigger STD of two sides as standard deviation, and the median of inliers as the mode.
+    Input arguments
+    ~~~~~~~~~~~~~~~
+        inVec (numpy.1darray): a float32 input vector of values to fit the model to
+        inWeight (numpy.1darray): a float32 input vector of weights for each data point, doesn't need to sum to 1
+        MSSE_LAMBDA : How far (normalized by STD of the Gaussian) from the 
+                        mean of the Gaussian, data is considered inlier.
+                        default: 3.0
+        optIters: number of iterations of FLKOS for this fitting
+            value 0: returns total mean and total STD
+            value 1: returns topKthPerc percentile and the scale by MSSE.
+            value 8 and above is recommended for optimization according 
+                    to Newton method
+            default : 12
+        topKthPerc: A rough but certain guess of portion of inliers, between 0 and 1, e.g. 0.5. 
+                    Choose the topKthPerc to be as high as you are sure the portion of data is inlier.
+                    if you are not sure at all, refer to the note above this code.
+                    default : 0.5
+        bottomKthPerc: We'd like to make a sample out of worst inliers from data points that are
+                       between bottomKthPerc and topKthPerc of sorted residuals.
+                       set it to 0.9*topKthPerc, if N is number of data points, then make sure that
+                       (topKthPerc - bottomKthPerc)*N>4, 
+                       it is best if bottomKthPerc*N>12 then MSSE makes sense
+                       otherwise the code may return non-robust results.
+        minimumResidual : minimum fitting error to initialize MSSE (dtype = 'float32')
+                          default : 0
+    Output
+    ~~~~~~
+        tuple of two numpy.1darrays, robust mode and standard deviation of the density from the longer tail
+    """
+    if(inWeights is None):
+        inWeights = np.ones(inVec.shape, dtype='float32')
+    modelParams = np.zeros(2, dtype='float32')
+    RGFCLib.medianOfFits((inVec.copy()).astype('float32'),
+                         (inWeights.copy()).astype('float32'),
+                         modelParams, 
+                         modelValueInit,
+                         inVec.shape[0],
+                         topkMin,
+                         topkMax,
+                         numSamples,
+                         samplePerc,
+                         MSSE_LAMBDA,
+                         optIters,
+                         minimumResidual)
+    return np.array([modelParams[0], modelParams[1]])
+
+
 def fitValueTensor(inTensor,
                    inMask = None,
                    topKthPerc = 0.5,
@@ -390,7 +454,8 @@ def fitBackground(inImage,
                   stretch2CornersOpt = 0,
                   numModelParams = 4,
                   optIters = 12,
-                  numStrides = 0):
+                  numStrides = 0,
+                  minimumResidual = 0):
     """ fit a plane to the background of the image uainf convolving the window by number of strides
         and calculate the value of the background plane and STD at the location of each pixel.
     
@@ -427,6 +492,7 @@ def fitBackground(inImage,
                     and winX and Y are 16 and numStrides is 1, from 0 to 15 and 15 to 31,
                     will be analysed. But if numStrides is 2, from 0 to 15, 10 to 25 and 15 to 31
                     will be analysed and averaged and so on ...
+        minimumResidual: Minimum fitting error if available
 
     Output
     ~~~~~~
@@ -458,7 +524,8 @@ def fitBackground(inImage,
                      MSSE_LAMBDA,
                      stretch2CornersOpt,
                      numModelParams,
-                     optIters)
+                     optIters,
+                     minimumResidual)
     
     _sums = np.ones((2, n_R, n_C), dtype='uint8')
     if(numStrides>1):
@@ -481,7 +548,8 @@ def fitBackground(inImage,
                                  MSSE_LAMBDA,
                                  stretch2CornersOpt,
                                  numModelParams,
-                                 optIters)
+                                 optIters,
+                                 minimumResidual)
                 bckParam[:,wSDRow:n_R-winX+wSDRow ,wSDClm:n_C-winY+wSDClm] += modelParamsMap
                 _sums[:,wSDRow:n_R-winX+wSDRow ,wSDClm:n_C-winY+wSDClm] += 1
     return(bckParam / _sums)
@@ -496,7 +564,8 @@ def fitBackgroundTensor(inImage_Tensor,
                         stretch2CornersOpt = 0,
                         numModelParams = 4,
                         optIters = 12,
-                        numStrides = 0):
+                        numStrides = 0,
+                        minimumResidual = 0):
     """ fit a plane by convolving the model to each image in the input Tensor and report background values and STD for each pixel for each plane
     
     Input arguments
@@ -527,6 +596,7 @@ def fitBackgroundTensor(inImage_Tensor,
                     and winX and Y are 16 and numStrides is 1, from 0 to 15 and 15 to 31,
                     will be analysed. But if numStrides is 2, from 0 to 15, 10 to 25 and 15 to 31
                     will be analysed and averaged. This means that the method will run 7 times.
+        minimumResidual : minimum fitting error if available
     Output
     ~~~~~~
         2 x n_F x n_R x n_C where out[0] would be background mean and out[1] would be STD for each pixel in the Tensor.
@@ -560,11 +630,12 @@ def fitBackgroundTensor(inImage_Tensor,
                                     MSSE_LAMBDA,
                                     stretch2CornersOpt,
                                     numModelParams,
-                                    optIters)
+                                    optIters,
+                                    minimumResidual)
 
     bckParam = np.array([model_mean, model_std])    
     _sums = np.ones((2, n_F, n_R, n_C), dtype='uint8')
-    if(numStrides>1):
+    if(numStrides>0):
         wSDListRows = np.linspace(0, winX, numStrides+2, dtype='uint8')[1:-1]
         wSDListClms = np.linspace(0, winY, numStrides+2, dtype='uint8')[1:-1]
         for wSDRow in wSDListRows:
@@ -589,7 +660,8 @@ def fitBackgroundTensor(inImage_Tensor,
                                                  MSSE_LAMBDA,
                                                  stretch2CornersOpt,
                                                  numModelParams,
-                                                 optIters)
+                                                 optIters,
+                                                 minimumResidual)
     
                 bckParam[:,:,wSDRow:n_R-winX+wSDRow ,wSDClm:n_C-winY+wSDClm] += np.array([model_mean, model_std])
                 _sums[:,:,wSDRow:n_R-winX+wSDRow ,wSDClm:n_C-winY+wSDClm] += 1
@@ -606,7 +678,9 @@ def fitBackgroundRadially(inImage,
                           MSSE_LAMBDA = 3.0,
                           optIters = 12,
                           numStrides = 0,
-                          finiteSampleBias = 400):
+                          finiteSampleBias = 200,
+                          minimumResidual = 0,
+                          return_vecMP = False):
     """ fit a value to the ring around the image and fine tune it by convolving the resolution shells
         by number of stride and calculate the value of the background of the ring
         and STD at the location of each pixel.
@@ -643,10 +717,16 @@ def fitBackgroundRadially(inImage,
                        otherwise the code may return non-robust results.
         numStrides: by giving a shellWidth>1, one can desire convolving the shell over radius by
             number of strides.
+        minimumResidual: the minimum residual for MSSE to initialize with, similar to RANSAC,
+            default: 0
+        return_vecMP: True if you'd like to receive a vector with radial info as a vector
+            
 
     Output
     ~~~~~~
-        numpy array with 2 parameters for each pixel : 2 x n_R, n_C : Rmean and RSTD.
+        modelParametersMap: numpy array with 2 parameters for each pixel : 2 x n_R, n_C : Rmean and RSTD. In case return_vecMP is set to True, you get a tuple in the output, the first element will be numpy array modelParametersMap which is the same size as the image and 
+        the second element will be a vector of size the maximum radius with two elements,
+        average and standard deviation.
     """
     
     if(inMask is None):
@@ -657,15 +737,21 @@ def fitBackgroundRadially(inImage,
 
     n_R = inImage.shape[0]
     n_C = inImage.shape[1]
-            
+
+    vecMP = np.zeros((2, int(np.ceil(np.sqrt(n_R*n_R/4+n_C*n_C/4)))), dtype='float32')
+    _vecMP = np.zeros((2, int(np.ceil(np.sqrt(n_R*n_R/4+n_C*n_C/4)))), dtype='float32')
+
     bckParam = np.zeros((2, n_R, n_C), dtype='float32')
+    if(shellWidth<1):
+        shellWidth = 1
     modelParamsMap = bckParam.copy()
-    _sums = np.zeros((2, n_R, n_C), dtype='uint8')
-    shellStrideList = np.linspace(0, shellWidth, numStrides+1, dtype='uint8')[:-1]
+    _sums = 0
+    shellStrideList = np.linspace(0, shellWidth, numStrides + 1, dtype='uint8')
     for shellStride in shellStrideList:
         RGFCLib.fitBackgroundRadially(inImage.astype('float32'),
                                       inMask,
                                       modelParamsMap,
+                                      _vecMP,
                                       minRes+shellStride,
                                       maxRes,
                                       shellWidth,
@@ -676,7 +762,33 @@ def fitBackgroundRadially(inImage,
                                       topKthPerc,
                                       bottomKthPerc,
                                       MSSE_LAMBDA,
-                                      optIters)
+                                      optIters,
+                                      minimumResidual)
         bckParam += modelParamsMap
+        vecMP += _vecMP
         _sums += 1
-    return(bckParam / _sums)
+    if(return_vecMP):
+        return(bckParam/_sums, vecMP/_sums)
+    else:
+        return(bckParam/_sums)
+
+def fitValue_by_meanShift(inVec, minSNR = 3.0, MS_numIter = 8):
+    MS_mu = inVec.mean()
+    MS_std = inVec.std()
+    for _ in range(MS_numIter):
+        rth = MS_mu + minSNR*MS_std
+        lth = MS_mu - minSNR*MS_std
+        vec = inVec[(lth<inVec) & (inVec<rth)].copy()
+        MS_mu = vec.mean()
+        MS_std = vec.std()
+    return(np.array([MS_mu, MS_std]))
+
+def fitPlane_by_meanShift(inX, inY, inZ, minSNR = 3.0, MS_numIter = 8):
+    res2 = 0*inX.copy()
+    for _ in range(MS_numIter):
+        mP = fitPlane(inX[res2<=minSNR**2],
+                      inY[res2<=minSNR**2],
+                      inZ[res2<=minSNR**2],
+                      1, 0, MSSE_LAMBDA=0)
+        res2 = (inZ - (mP[0]*inX + mP[1]*inY + mP[2]))**2/mP[3]**2
+    return(mP)
