@@ -79,6 +79,21 @@ RGFCLib.fitValue2Skewed.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 ctypes.c_float, ctypes.c_uint32, ctypes.c_float, 
                 ctypes.c_float, ctypes.c_float, ctypes.c_uint8, ctypes.c_float]                
+
+'''
+void medianOfFits(float *vec, float *weights, 
+                  float *modelParams, float theta, unsigned int N,
+                  float topkMin, float topkMax, unsigned int numSamples, float samplePerc,
+                  float MSSE_LAMBDA, unsigned char optIters, float minimumResidual) 
+'''
+RGFCLib.medianOfFits.argtypes = [
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                ctypes.c_float, ctypes.c_uint32, 
+                ctypes.c_float, ctypes.c_float, ctypes.c_uint32, ctypes.c_float,   
+                ctypes.c_float, ctypes.c_uint8, ctypes.c_float]                
+
 '''
 void RobustAlgebraicLineFitting(float* x, float* y, float* mP, unsigned int N,
 							  float topKthPerc, float bottomKthPerc, float MSSE_LAMBDA)
@@ -138,7 +153,8 @@ void RSGImage(float* inImage, unsigned char* inMask, float *modelParamsMap,
 				unsigned int X, unsigned int Y,
 				float topkPerc, float botkPerc,
 				float MSSE_LAMBDA, unsigned char stretch2CornersOpt,
-				unsigned char numModelParams, unsigned char optIters)
+				unsigned char numModelParams, unsigned char optIters,
+                float minimumResidual)
 '''                
 RGFCLib.RSGImage.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
@@ -148,7 +164,7 @@ RGFCLib.RSGImage.argtypes = [
                 ctypes.c_uint32, ctypes.c_uint32, 
                 ctypes.c_float, ctypes.c_float, 
                 ctypes.c_float, ctypes.c_uint8, 
-                ctypes.c_uint8, ctypes.c_uint8]
+                ctypes.c_uint8, ctypes.c_uint8, ctypes.c_float]
        
 '''
 void RSGImage_by_Image_Tensor(float* inImage_Tensor, unsigned char* inMask_Tensor,
@@ -157,7 +173,8 @@ void RSGImage_by_Image_Tensor(float* inImage_Tensor, unsigned char* inMask_Tenso
 						unsigned int N, unsigned int X, unsigned int Y,
 						float topkPerc, float botkPerc,
 						float MSSE_LAMBDA, unsigned char stretch2CornersOpt,
-						unsigned char numModelParams, unsigned char optIters)
+						unsigned char numModelParams, unsigned char optIters,
+                        float minimumResidual)
 '''
 RGFCLib.RSGImage_by_Image_Tensor.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
@@ -168,11 +185,11 @@ RGFCLib.RSGImage_by_Image_Tensor.argtypes = [
                 ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32,
                 ctypes.c_float, ctypes.c_float, 
                 ctypes.c_float, ctypes.c_uint8, 
-                ctypes.c_uint8, ctypes.c_uint8]
+                ctypes.c_uint8, ctypes.c_uint8, ctypes.c_float]
 
 '''
 void fitBackgroundRadially(float* inImage, unsigned char* inMask, 
-						   float* modelParamsMap,
+						   float* modelParamsMap, float* vecMP,
  						   unsigned int minRes, 
 						   unsigned int maxRes, 
 						   unsigned int shellWidth,
@@ -181,13 +198,16 @@ void fitBackgroundRadially(float* inImage, unsigned char* inMask,
 						   unsigned int X, unsigned int Y,
 						   float topkPerc, float botkPerc, 
 						   float MSSE_LAMBDA, 
-						   unsigned char optIters);
+						   unsigned char optIters,
+						   float minimumResidual);
 '''
 RGFCLib.fitBackgroundRadially.argtypes = [
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_uint8, flags='C_CONTIGUOUS'),
                 np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
+                np.ctypeslib.ndpointer(ctypes.c_float, flags='C_CONTIGUOUS'),
                 ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32, 
                 ctypes.c_uint8, ctypes.c_uint32, 
                 ctypes.c_uint32, ctypes.c_uint32, 
-                ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_uint8]
\ No newline at end of file
+                ctypes.c_float, ctypes.c_float, 
+                ctypes.c_float, ctypes.c_uint8, ctypes.c_float]
\ No newline at end of file
